﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using AKS.BusinessModel;
using AKS.BusinessModel.Inventory;

namespace AKS.ViewModel
{
  /// <summary>
  /// Bindungsliste für Projekte
  /// </summary>
  public class ProjectBindingList : BindingList<Project>
  {
    /// <summary>
    /// Collections für das Change-Tracking
    /// </summary>
    private Projects inserts = new Projects();
    private Projects deletes = new Projects();

    #region Internal Constructors

    /// <summary>
    /// Konstruktor. Fügt die Projekte in diese Bindinglist ein
    /// </summary>
    /// <param name="projects">Liste aller Projekte</param>
    public ProjectBindingList( Projects projects )
      : this( projects.Values.ToList<Project>() )
    {
    }

    /// <summary>
    /// Konstruktor. Fügt die Projekte in diese Bindinglist ein
    /// </summary>
    /// <param name="projects">Liste aller Projekte</param>
    public ProjectBindingList( IList<Project> projects )
      : base()
    {
      // Projekte in die Bindinglist übernehmen
      foreach ( Project project in projects )
        this.Add( project );

      // Eventhandler für die Behandlung neuer und geänderter Elemente
      this.ListChanged += new ListChangedEventHandler( ProjectBindingListListChanged );
    }

    #endregion

    #region Public Properties

    /// <summary>
    /// Liste der neuen Projekte
    /// </summary>
    public Projects Inserts
    {
      get { return this.inserts; }
    }

    /// <summary>
    /// Liste der gelöschten Projekte
    /// </summary>
    public Projects Deletes
    {
      get { return this.deletes; }
    }

    #endregion

    #region Protected Methods

    /// <summary>
    /// Change-Tracking für neue Projekte.
    /// </summary>
    /// <remarks>Das Change-Tracking für gelöschte Projekte wird separat behandelt, da 
    /// das Element beim Aufruf dieser Funktion bereits aus der Liste entfernt ist</remarks>
    /// <param name="sender">Dieses Objekt</param>
    /// <param name="e">Informationen über das geänderte Listenelement</param>
    protected void ProjectBindingListListChanged( object sender, ListChangedEventArgs e )
    {
      // Neues Element: in Liste der neuen Projekte einfügen
      if ( e.ListChangedType == ListChangedType.ItemAdded )
        this.inserts.Add( this[e.NewIndex] );

    }

    /// <summary>
    /// Change-Tracking für gelöschte Projekte
    /// </summary>
    /// <param name="index">Index des Projekts das gelöscht wird.</param>
    protected override void RemoveItem( int index )
    {
      Project deletedProject = this[index];

      // wenn das Element bereits existierte (d.h. es ist nicht in der Inserts-Liste), dann in Lösch-Liste eintragen
      if ( !this.inserts.ContainsKey( deletedProject.Id ) )
        this.deletes.Add( this[index] );

      // ggf. aus Einfüge- entfernen
      this.inserts.Remove( deletedProject.Id );

      base.RemoveItem( index );
    }

    #endregion


  }
}
